@@ -39,7 +39,7 @@ class Config:
         self.request_retries: int = 6
         self.request_retry_time: int = 10
         self.scheduler_refresh_rate: int = 10
-        self.default_timeout: float = 3600.
+        self.default_timeout: float = 14400.
 
 
 config = Config()
@@ -95,6 +95,8 @@ from previsionio.deployed_model import DeployedModel
 from previsionio.experiment_deployment import ExperimentDeployment
 from previsionio.prediction import ValidationPrediction, DeploymentPrediction
 
+from previsionio.pipeline import PipelineScheduledRun
+
 __all__ = [
     'client',
     'AdvancedModel',
@@ -139,7 +141,8 @@ __all__ = [
     'Experiment',
     'ExperimentDeployment',
     'ValidationPrediction',
-    'DeploymentPrediction'
+    'DeploymentPrediction',
+    'PipelineScheduledRun'
 ]
 
 from . import _version
