@@ -46,7 +46,6 @@ class ExternalExperimentVersion(ClassicExperimentVersion):
         # this is dict, maybe we should parse it in a tuple like in creation
         self.external_models = experiment_version_info.get('external_models')
 
-        self.metric = experiment_version_info.get('metric')
         self.dataset_id: Union[str, None] = experiment_version_info.get('dataset_id')
 
         self.metric: str = experiment_version_params['metric']
@@ -250,9 +249,6 @@ class ExternalExperimentVersion(ClassicExperimentVersion):
     def get_cv(self):
         raise NotImplementedError
 
-    def predict_single(self, data) -> Dict:
-        raise NotImplementedError
-
     def predict_from_dataset(self,
                              dataset,
                              dataset_folder=None) -> pd.DataFrame:
